@@ -1,4 +1,4 @@
-# Copyright (C) 2000-2003 by the Free Software Foundation, Inc.
+# Copyright (C) 2000-2016 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,10 +12,12 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+# USA.
 
 import sys
 import time
+import locale
 import gettext
 from types import StringType, UnicodeType
 
@@ -24,6 +26,16 @@ from Mailman.SafeDict import SafeDict
 
 _translation = None
 
+
+def _get_ctype_charset():
+    old = locale.setlocale(locale.LC_CTYPE, '')
+    charset = locale.nl_langinfo(locale.CODESET)
+    locale.setlocale(locale.LC_CTYPE, old)
+    return charset
+
+if not mm_cfg.DISABLE_COMMAND_LOCALE_CSET:
+    _ctype_charset = _get_ctype_charset()
+
 
 
 def set_language(language=None):
@@ -53,7 +65,7 @@ if _translation is None:
 
 
 
-def _(s):
+def _(s, frame=1):
     if s == '':
         return s
     assert s
@@ -69,7 +81,7 @@ def _(s):
     # original string is 1) locals dictionary, 2) globals dictionary.
     #
     # First, get the frame of the caller
-    frame = sys._getframe(1)
+    frame = sys._getframe(frame)
     # A `safe' dictionary is used so we won't get an exception if there's a
     # missing key in the dictionary.
     dict = SafeDict(frame.f_globals.copy())
@@ -86,9 +98,30 @@ def _(s):
     for k, v in dict.items():
         if isinstance(v, UnicodeType):
             dict[k] = v.encode(charset, 'replace')
-    return tns % dict
+    try:
+        return tns % dict
+    except (ValueError, TypeError):
+        # Bad interpolation format. Punt.
+        return tns
 
 
+
+def tolocale(s):
+    global _ctype_charset
+    if isinstance(s, UnicodeType):
+        return s
+    source = _translation.charset ()
+    if not source:
+        return s
+    return unicode(s, source, 'replace').encode(_ctype_charset, 'replace')
+
+if mm_cfg.DISABLE_COMMAND_LOCALE_CSET:
+    C_ = _
+else:
+    def C_(s):
+        return tolocale(_(s, 2))
+
+    
 
 def ctime(date):
     # Don't make these module globals since we have to do runtime translation
@@ -107,7 +140,12 @@ def ctime(date):
     if isinstance(date, StringType):
         try:
             year, mon, day, hh, mm, ss, wday, ydat, dst = time.strptime(date)
-            tzname = time.tzname[dst and 1 or 0]
+            if dst in (0,1):
+                tzname = time.tzname[dst]
+            else:
+                # MAS: No exception but dst = -1 so try
+                return ctime(time.mktime((year, mon, day, hh, mm, ss, wday,
+                                          ydat, dst)))
         except (ValueError, AttributeError):
             try:
                 wday, mon, day, hms, year = date.split()
@@ -132,7 +170,8 @@ def ctime(date):
                         break
     else:
         year, mon, day, hh, mm, ss, wday, yday, dst = time.localtime(date)
-        tzname = time.tzname[dst and 1 or 0]
+        if dst in (0,1):
+            tzname = time.tzname[dst]
 
     wday = daysofweek[wday]
     mon = months[mon]
